@@ -1,11 +1,11 @@
 """Tests for the redis orm"""
 
-from typing import Dict, Any
+from typing import Dict, Any, List
 
 import pytest
 
 from pydantic_redis.config import RedisConfig
-from pydantic_redis.model import Model
+from pydantic_redis.model import Model, strip_leading, NESTED_MODEL_PREFIX
 from test.conftest import (
     redis_store_fixture,
     Book,
@@ -135,13 +135,64 @@ def test_insert_single_nested(store):
 
 @pytest.mark.parametrize("store", redis_store_fixture)
 def test_update_nested_list_of_models(store):
-    Library.insert(libraries)
+    data = [Library(name="Babel Library", address="In a book", books=books)]
+    Library.insert(data)
+    # the list of nested models is automatically inserted
     got = sorted(Book.select(), key=lambda x: x.title)
     expected = sorted(books, key=lambda x: x.title)
     assert expected == got
 
     got = sorted(Library.select(), key=lambda x: x.name)
-    expected = sorted(libraries, key=lambda x: x.name)
+    expected = sorted(data, key=lambda x: x.name)
+    assert got == expected
+
+
+@pytest.mark.parametrize("store", redis_store_fixture)
+def test_update_optional_nested_list_of_models(store):
+    data = [Library(name="Babel Library", address="In a book", lost=books)]
+    Library.insert(data)
+    # the list of nested models is automatically inserted
+    got = sorted(Book.select(), key=lambda x: x.title)
+    expected = sorted(books, key=lambda x: x.title)
+    assert expected == got
+
+    got = sorted(Library.select(), key=lambda x: x.name)
+    expected = sorted(data, key=lambda x: x.name)
+    assert got == expected
+
+
+@pytest.mark.parametrize("store", redis_store_fixture)
+def test_update_nested_tuple_of_models(store):
+    jane = authors["jane"]
+    new_stuff = (books[0], jane, books[1], 8)
+    data = [Library(name="Babel Library", address="In a book", new=new_stuff)]
+    Library.insert(data)
+    # the tuple of nested models is automatically inserted
+    got = sorted(Book.select(), key=lambda x: x.title)
+    expected = sorted([books[0], books[1]], key=lambda x: x.title)
+    assert expected == got
+
+    got = sorted(Author.select(), key=lambda x: x.name)
+    expected = sorted([books[0].author, jane], key=lambda x: x.name)
+    assert expected == got
+
+    got = sorted(Library.select(), key=lambda x: x.name)
+    expected = sorted(data, key=lambda x: x.name)
+    assert got == expected
+
+
+@pytest.mark.parametrize("store", redis_store_fixture)
+def test_update_optional_nested_tuple_of_models(store):
+    popular_books = (books[0], books[2])
+    data = [Library(name="Babel Library", address="In a book", popular=popular_books)]
+    Library.insert(data)
+    # the tuple of nested models is automatically inserted
+    got = sorted(Book.select(), key=lambda x: x.title)
+    expected = sorted(popular_books, key=lambda x: x.title)
+    assert expected == got
+
+    got = sorted(Library.select(), key=lambda x: x.name)
+    expected = sorted(data, key=lambda x: x.name)
     assert got == expected
 
 
@@ -302,8 +353,12 @@ def test_delete_multiple(store):
         assert expected == authors_in_redis_as_models
 
 
-def __deserialize_book_data(raw_book_data: Dict[bytes, Any]) -> Book:
+def __deserialize_book_data(raw_book_data: Dict[str, Any]) -> Book:
     """Deserializes the raw book data returning a book instance"""
+    author_id = raw_book_data.pop(f"{NESTED_MODEL_PREFIX}author")
+    author_id = strip_leading(author_id, "author_%&_")
+
     data = Book.deserialize_partially(raw_book_data)
-    data["author"] = Author.select(ids=[data["__author"]])[0]
+
+    data["author"] = Author.select(ids=[author_id])[0]
     return Book(**data)
